This is a live mirror of the Perl 5 development currently hosted at https://github.com/perl/perl5
Bump NEXT version to 0.61 for CPAN release
[perl5.git] / opcode.pl
index 098f83c..7f88036 100755 (executable)
--- a/opcode.pl
+++ b/opcode.pl
@@ -8,11 +8,9 @@ BEGIN {
 
 my $opcode_new = 'opcode.h-new';
 my $opname_new = 'opnames.h-new';
-open(OC, ">$opcode_new") || die "Can't create $opcode_new: $!\n";
-binmode OC;
-open(ON, ">$opname_new") || die "Can't create $opname_new: $!\n";
-binmode ON;
-select OC;
+my $oc = safer_open($opcode_new);
+my $on = safer_open($opname_new);
+select $oc;
 
 # Read data.
 
@@ -128,7 +126,7 @@ PERL_PPDEF(Perl_unimplemented_op)
 
 END
 
-print ON <<"END";
+print $on <<"END";
 /* -*- buffer-read-only: t -*-
  *
  *    opnames.h
@@ -150,14 +148,14 @@ END
 
 my $i = 0;
 for (@ops) {
-    # print ON "\t", &tab(3,"OP_\U$_,"), "/* ", $i++, " */\n";
-      print ON "\t", &tab(3,"OP_\U$_"), " = ", $i++, ",\n";
+    # print $on "\t", &tab(3,"OP_\U$_,"), "/* ", $i++, " */\n";
+      print $on "\t", &tab(3,"OP_\U$_"), " = ", $i++, ",\n";
 }
-print ON "\t", &tab(3,"OP_max"), "\n";
-print ON "} opcode;\n";
-print ON "\n#define MAXO ", scalar @ops, "\n";
-print ON "#define OP_phoney_INPUT_ONLY -1\n";
-print ON "#define OP_phoney_OUTPUT_ONLY -2\n\n";
+print $on "\t", &tab(3,"OP_max"), "\n";
+print $on "} opcode;\n";
+print $on "\n#define MAXO ", scalar @ops, "\n";
+print $on "#define OP_phoney_INPUT_ONLY -1\n";
+print $on "#define OP_phoney_OUTPUT_ONLY -2\n\n";
 
 # Emit op names and descriptions.
 
@@ -344,6 +342,7 @@ my %opflags = (
 
 my %OP_IS_SOCKET;
 my %OP_IS_FILETEST;
+my %OP_IS_FT_ACCESS;
 my $OCSHIFT = 9;
 my $OASHIFT = 13;
 
@@ -352,12 +351,12 @@ for my $op (@ops) {
     my $flags = $flags{$op};
     for my $flag (keys %opflags) {
        if ($flags =~ s/$flag//) {
-           die "Flag collision for '$op' ($flags{$op}, $flag)"
+           die "Flag collision for '$op' ($flags{$op}, $flag)\n"
                if $argsum & $opflags{$flag};
            $argsum |= $opflags{$flag};
        }
     }
-    die qq[Opcode '$op' has no class indicator ($flags{$op} => $flags)]
+    die qq[Opcode '$op' has no class indicator ($flags{$op} => $flags)\n]
        unless exists $opclass{$flags};
     $argsum |= $opclass{$flags} << $OCSHIFT;
     my $argshift = $OASHIFT;
@@ -366,6 +365,7 @@ for my $op (@ops) {
            # record opnums of these opnames
            $OP_IS_SOCKET{$op}   = $opnum{$op} if $arg =~ s/s//;
            $OP_IS_FILETEST{$op} = $opnum{$op} if $arg =~ s/-//;
+           $OP_IS_FT_ACCESS{$op} = $opnum{$op} if $arg =~ s/\+//;
         }
        my $argnum = ($arg =~ s/\?//) ? 8 : 0;
         die "op = $op, arg = $arg\n"
@@ -393,7 +393,7 @@ END
 
 # Emit OP_IS_* macros
 
-print ON <<EO_OP_IS_COMMENT;
+print $on <<EO_OP_IS_COMMENT;
 
 /* the OP_IS_(SOCKET|FILETEST) macros are optimized to a simple range
     check because all the member OPs are contiguous in opcode.pl
@@ -403,6 +403,7 @@ EO_OP_IS_COMMENT
 
 gen_op_is_macro( \%OP_IS_SOCKET, 'OP_IS_SOCKET');
 gen_op_is_macro( \%OP_IS_FILETEST, 'OP_IS_FILETEST');
+gen_op_is_macro( \%OP_IS_FT_ACCESS, 'OP_IS_FILETEST_ACCESS');
 
 sub gen_op_is_macro {
     my ($op_is, $macname) = @_;
@@ -414,46 +415,42 @@ sub gen_op_is_macro {
        } keys %$op_is;
        
        my $last = pop @rest;   # @rest slurped, get its last
-       
+       die "Invalid range of ops: $first .. $last\n" unless $last;
+
+       print $on "#define $macname(op) \\\n\t(";
+
        # verify that op-ct matches 1st..last range (and fencepost)
        # (we know there are no dups)
        if ( $op_is->{$last} - $op_is->{$first} == scalar @rest + 1) {
            
            # contiguous ops -> optimized version
-           print ON "#define $macname(op)      \\\n\t(";
-           print ON "(op) >= OP_" . uc($first) . " && (op) <= OP_" . uc($last);
-           print ON ")\n\n";
+           print $on "(op) >= OP_" . uc($first) . " && (op) <= OP_" . uc($last);
+           print $on ")\n\n";
        }
        else {
-           print ON "\n#define $macname(op)    \\\n\t(";
-           print ON join(" || \\\n\t ",
-                         map { "(op) == OP_" . uc() } sort keys %OP_IS_SOCKET);
-           print ON ")\n\n";
+           print $on join(" || \\\n\t ",
+                         map { "(op) == OP_" . uc() } sort keys %$op_is);
+           print $on ")\n\n";
        }
     }
 }
 
-print OC "/* ex: set ro: */\n";
-print ON "/* ex: set ro: */\n";
+print $oc "/* ex: set ro: */\n";
+print $on "/* ex: set ro: */\n";
 
-close OC or die "Error closing opcode.h: $!";
-close ON or die "Error closing opnames.h: $!";
+safer_close($oc);
+safer_close($on);
 
-foreach ('opcode.h', 'opnames.h') {
-    safer_rename_silent $_, "$_-old";
-}
-safer_rename $opcode_new, 'opcode.h';
-safer_rename $opname_new, 'opnames.h';
+rename_if_different $opcode_new, 'opcode.h';
+rename_if_different $opname_new, 'opnames.h';
 
 my $pp_proto_new = 'pp_proto.h-new';
 my $pp_sym_new  = 'pp.sym-new';
 
-open PP, ">$pp_proto_new" or die "Error creating $pp_proto_new: $!";
-binmode PP;
-open PPSYM, ">$pp_sym_new" or die "Error creating $pp_sym_new: $!";
-binmode PPSYM;
+my $pp = safer_open($pp_proto_new);
+my $ppsym = safer_open($pp_sym_new);
 
-print PP <<"END";
+print $pp <<"END";
 /* -*- buffer-read-only: t -*-
    !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
    This file is built by opcode.pl from its data.  Any changes made here
@@ -462,7 +459,7 @@ print PP <<"END";
 
 END
 
-print PPSYM <<"END";
+print $ppsym <<"END";
 # -*- buffer-read-only: t -*-
 #
 # !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
@@ -474,30 +471,27 @@ END
 
 
 for (sort keys %ckname) {
-    print PP "PERL_CKDEF(Perl_$_)\n";
-    print PPSYM "Perl_$_\n";
+    print $pp "PERL_CKDEF(Perl_$_)\n";
+    print $ppsym "Perl_$_\n";
 #OP *\t", &tab(3,$_),"(OP* o);\n";
 }
 
-print PP "\n\n";
+print $pp "\n\n";
 
 for (@ops) {
     next if /^i_(pre|post)(inc|dec)$/;
     next if /^custom$/;
-    print PP "PERL_PPDEF(Perl_pp_$_)\n";
-    print PPSYM "Perl_pp_$_\n";
+    print $pp "PERL_PPDEF(Perl_pp_$_)\n";
+    print $ppsym "Perl_pp_$_\n";
 }
-print PP "\n/* ex: set ro: */\n";
-print PPSYM "\n# ex: set ro:\n";
+print $pp "\n/* ex: set ro: */\n";
+print $ppsym "\n# ex: set ro:\n";
 
-close PP or die "Error closing pp_proto.h: $!";
-close PPSYM or die "Error closing pp.sym: $!";
+safer_close($pp);
+safer_close($ppsym);
 
-foreach ('pp_proto.h', 'pp.sym') {
-    safer_rename_silent $_, "$_-old";
-}
-safer_rename $pp_proto_new, 'pp_proto.h';
-safer_rename $pp_sym_new, 'pp.sym';
+rename_if_different $pp_proto_new, 'pp_proto.h';
+rename_if_different $pp_sym_new, 'pp.sym';
 
 END {
   foreach ('opcode.h', 'opnames.h', 'pp_proto.h', 'pp.sym') {
@@ -602,7 +596,9 @@ __END__
 # Values for the operands are:
 # scalar      - S            list     - L            array     - A
 # hash        - H            sub (CV) - C            file      - F
-# socket      - Fs           filetest - F-           reference - R
+# socket      - Fs           filetest - F-           filetest_access - F-+
+
+# reference - R
 # "?" denotes an optional operand.
 
 # Nothing.
@@ -767,7 +763,7 @@ vec         vec                     ck_fun          ist@    S S S
 index          index                   ck_index        isT@    S S S?
 rindex         rindex                  ck_index        isT@    S S S?
 
-sprintf                sprintf                 ck_fun          mst@    S L
+sprintf                sprintf                 ck_fun          fmst@   S L
 formline       formline                ck_fun          ms@     S L
 ord            ord                     ck_fun          ifsTu%  S?
 chr            chr                     ck_fun          fsTu%   S?
@@ -948,12 +944,12 @@ getpeername       getpeername             ck_fun          is%     Fs
 
 lstat          lstat                   ck_ftst         u-      F
 stat           stat                    ck_ftst         u-      F
-ftrread                -R                      ck_ftst         isu-    F-
-ftrwrite       -W                      ck_ftst         isu-    F-
-ftrexec                -X                      ck_ftst         isu-    F-
-fteread                -r                      ck_ftst         isu-    F-
-ftewrite       -w                      ck_ftst         isu-    F-
-fteexec                -x                      ck_ftst         isu-    F-
+ftrread                -R                      ck_ftst         isu-    F-+
+ftrwrite       -W                      ck_ftst         isu-    F-+
+ftrexec                -X                      ck_ftst         isu-    F-+
+fteread                -r                      ck_ftst         isu-    F-+
+ftewrite       -w                      ck_ftst         isu-    F-+
+fteexec                -x                      ck_ftst         isu-    F-+
 ftis           -e                      ck_ftst         isu-    F-
 ftsize         -s                      ck_ftst         istu-   F-
 ftmtime                -M                      ck_ftst         stu-    F-
@@ -1052,6 +1048,7 @@ semctl            semctl                  ck_fun          imst@   S S S S
 
 require                require                 ck_require      du%     S?
 dofile         do "file"               ck_fun          d1      S
+hintseval      eval hints              ck_svconst      s$
 entereval      eval "string"           ck_eval         d%      S
 leaveeval      eval "string" exit      ck_null         1       S
 #evalonce      eval constant string    ck_null         d1      S